-- NOTICE: This file is borrowed from Gediminas Liktaras year 2009 submission.


--------------------------------------------------------------------------------
-- | 
-- Module: Colors. 
--
-- Contains definitions of color types and some predefined colors. So far only 
-- the colors in RGB palette are defined.
--
-- If you need to convert colors to 'ColorVector' type, use the appropriate
-- functions in Ray module.
--
--------------------------------------------------------------------------------

module Colors (
    -- * Colors
    ColorRGB(..),

    makeRGB,        -- :: Int -> Int -> Int -> ColorRGB
    color2tuple,    -- :: ColorRGB -> (Int, Int, Int)

    -- * Predefined colors.
    black,
    blue,
    cyan,
    gray,
    green,
    magenta,
    red,
    yellow,
    white
    ) where


--------------------------------------------------------------------------------
-- RGB COLORS
--------------------------------------------------------------------------------

-- | A type, representing a color in RGB palette.
data ColorRGB = RGB Int Int Int
                deriving (Eq, Read, Show)

-- | Creates a color in RGB palette. 
makeRGB :: Int -> Int -> Int -> ColorRGB
makeRGB red green blue
    | red   < 0 || red   > 255 = error "Invalid RGB color component value."
    | green < 0 || green > 255 = error "Invalid RGB color component value."
    | blue  < 0 || blue  > 255 = error "Invalid RGB color component value."
    | otherwise                = RGB red green blue


-- | Converts a color into a tuple.
color2tuple :: ColorRGB -> (Int, Int, Int)
color2tuple (RGB r g b) = (r, g, b)


-- Predefined colors.
black   = makeRGB   0   0   0
blue    = makeRGB   0   0 255
cyan    = makeRGB   0 255 255
gray    = makeRGB 128 128 128
green   = makeRGB   0 255   0
magenta = makeRGB 255   0 255
red     = makeRGB 255   0   0
yellow  = makeRGB 255 255   0
white   = makeRGB 255 255 255

